schemaVersion: 2.2.2
metadata:
  name: ansible-demo
components:
  - name: tooling-container
    container:
      image: {{ dev_file_image }}
      memoryRequest: 256M
      memoryLimit: 6Gi
      cpuRequest: 250m
      cpuLimit: 2000m
      args: ["tail", "-f", "/dev/null"]
      env:
        - name: KUBEDOCK_ENABLED
          value: "true"
